"""
Copyright 2021-2022 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from logging import Logger
from typing import Union
from Setter.Common.ControllerPins import ControllerPins
from Setter.Common.SpiConfig import SpiConfig
from Setter.Common.UartConfig import UartConfig
from Setter.Exceptions.BoardConfigNotFoundException import BoardConfigNotFoundException
from Setter.Exceptions.BoardNotFoundException import BoardNotFoundException
from Setter.Logging import Logging


class Boards:
	"""
	Board pin configuration class.

	Methods
	-------
	get_model_config(model: str, config_type: str) -> Union[SpiConfig, UartConfig]:
		Return interface configuration for specified board.
	"""

	__models = {
		'IQD-GW-01': {
			'controller': ControllerPins(2, 0, 1, 12, 11),
			'spi': SpiConfig('/dev/spidev1.0', 19, 3, 10, True),
			'uart': UartConfig('/dev/ttyS1', 19, -1, 10, 57600, True)
		},
		'IQD-GW-02': {
			'controller': ControllerPins(2, 0, 1, 12, 11),
			'spi': SpiConfig('/dev/spidev1.0', 19, 3, -1, True, 7, 10, 6),
			'uart': UartConfig('/dev/ttyS1', 19, 3, -1, 57600, True, 7, 10, 6)
		},
		'IQD-GW04': {
			'controller': ControllerPins(25, 27, 17, 16, -1),
			'spi': SpiConfig('/dev/spidev1.0', 23, 22, -1, True, 18, 7, 6),
			'uart': UartConfig('/dev/ttyAMA1', 23, -1, -1, 57600, True, 18, 7, 6)
		},
		'Unipi Iris Zulu': {
			'controller': ControllerPins(504, 507, 506),
			'spi': SpiConfig('/dev/spidev1.0', 511, 510, -1, True, 509, 508, 505),
			'uart': UartConfig('/dev/ttymxc3', 511, 510, -1, 57600, True, 509, 508, 505)
		}
	}
	__logger: Logger = Logging.get_logger()

	@classmethod
	def get_model_config(cls, model: str, config_type: str) -> Union[SpiConfig, UartConfig]:
		"""
		Return interface configuration for specified board.

		Parameters
		----------
		model: str
			board model
		config_type: str
			configuration interface type

		Returns
		-------
		Union[SpiConfig, UartConfig]
			interface configuration object

		Exceptions
		----------
		BoardNotFoundException
			raised if specified board model is unknown
		BoardConfigNotFoundException
			raised if specified board does not have configuration for the required interface
		"""
		if model.startswith('IQD-GW-02'):
			model = 'IQD-GW-02'
		elif model.startswith('IQD-GW04'):
			model = 'IQD-GW04'
		if model not in cls.__models:
			error_str = f'Board model {model} not found.'
			cls.__logger.error(error_str)
			raise BoardNotFoundException(error_str)
		board = cls.__models[model]
		if config_type not in board:
			error_str = f'Board model {model} {config_type} pin configuration not found.'
			cls.__logger.error(error_str)
			raise BoardConfigNotFoundException(error_str)
		return board[config_type]

	@classmethod
	def get_controller_pins(cls, model: str) -> ControllerPins:
		"""
		Return controller configuration for specified board.

		Parameters
		----------
		model: str
			board model

		Returns
		-------
		ControllerPins
			controller pins configuration object

		Exceptions
		----------
		BoardNotFoundException
			raised if specified board model is unknown
		BoardConfigNotFoundException
			raised if specified board does not have controller pin configuration
		"""
		if model.startswith('IQD-GW-02'):
			model = 'IQD-GW-02'
		if model not in cls.__models:
			error_str = f'Board model {model} not found.'
			cls.__logger.error(error_str)
			raise BoardNotFoundException(error_str)
		board = cls.__models[model]
		if 'controller' not in board:
			error_str = f'Board model {model} controller pin configuration not found.'
			cls.__logger.error(error_str)
			raise BoardConfigNotFoundException(error_str)
		return board['controller']
