"""
Copyright 2021-2022 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import re

from Setter.Exceptions.ConfParserException import ConfParserException


class ConfParser:
	"""
	Configuration format parser class.

	Methods
	-------
	from_conf(content: str) -> dict:
		Convert conf/ini file content into dictionary.
	to_conf(content: dict) -> str:
		Convert dictionary into conf/ini format string.
	"""

	__header_pattern = '\\[[a-zA-Z0-9]+(-[a-zA-Z0-9]+)?'
	__option_pattern = '[^=]+=([^=]+)?'

	@classmethod
	def from_conf(cls, content: str) -> dict:
		"""
		Convert conf/ini file content into dictionary.

		Parameters
		----------
		content: str
			conf/ini file content string

		Returns
		-------
		dict
			dictionary containing conf/ini file content

		Exceptions
		----------
		ConfParserException
			raised if content has invalid conf syntax
		"""
		lines = content.splitlines(keepends=False)
		config = {}
		section = None
		for idx, line in enumerate(lines):
			# empty line
			if len(line) == 0:
				continue
			# ignore comments
			if line.startswith(('#', ';')):
				continue
			# section header
			if re.match(cls.__header_pattern, line):
				section = line[1:-1]
				config[section] = {}
			# option
			elif re.match(cls.__option_pattern, line):
				tokens = line.split('=', 1)
				config[section][tokens[0]] = tokens[1]
			else:
				raise ConfParserException(f'Failed to convert conf file to dictionary, file contains invalid content at line {idx + 1}.')
		return config

	@classmethod
	def to_conf(cls, content: dict) -> str:
		"""
		Convert dictionary into conf/ini format string.

		Parameters
		----------
		content: dict
			conf dictionary

		Returns
		-------
		str
			conf format string

		Exceptions
		----------
		ConfParserException
			raised if dict contains invalid property key value
		"""
		lines = []
		string = ''
		for key, value in content.items():
			if isinstance(value, dict) is dict:
				string += f'[{key}]\n'
				string += cls.to_conf(value) + '\n'
			else:
				try:
					str_value = str(value)
					lines.append(f'{key}={str_value}')
				except ValueError as value_error:
					raise ConfParserException(f'Failed to convert dictionary to conf, content of property {key} is invalid.') from value_error
		string += '\n'.join(lines)
		return string.strip()
