"""
Copyright 2021-2022 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import argparse
import os
import sys
import sentry_sdk

from sentry_sdk.integrations.atexit import AtexitIntegration
from Setter import __version__
from Setter.Common.AuxiliaryFunctions import AuxiliaryFunctions
from Setter.Common.Command import Command
from Setter.Common.GatewayFile import GatewayFile
from Setter.Enums.ErrorCodes import ErrorCodes
from Setter.Exceptions.BackupException import BackupException
from Setter.Exceptions.SetterBaseException import SetterBaseException
from Setter.ControllerConfig import ControllerConfig
from Setter.DaemonConfig import DaemonConfig
from Setter.GatewayConfig import GatewayConfig
from Setter.Logging import Logging
from Setter.UploaderConfig import UploaderConfig
from Setter.WebappConfig import WebappConfig


def main():
	"""Setter entry point."""
	parser = argparse.ArgumentParser(prog='IQRF Gateway Setter', description='Configuration tool for IQRF Gateway.')
	parser.add_argument(
		'-g', '--gateway', dest='gateway', action='store_true',
		help='Configures IQRF Gateway device for target platform. Also configures Gateway software.'
	)
	parser.add_argument(
		'-f', '--force', dest='force', action='store_true',
		help='Force re-configure IQRF Gateway device.'
	)
	parser.add_argument(
		'-c', '--controller', dest='controller', action='store_true',
		help='Configures IQRF Gateway Controller for the target platform.'
	)
	parser.add_argument(
		'-d', '--daemon', dest='daemon', action='store_true',
		help='Configures IQRF Gateway Daemon for target platform.'
	)
	parser.add_argument(
		'-u', '--uploader', dest='uploader', action='store_true',
		help='Configures IQRF Gateway Uploader for target platform.'
	)
	parser.add_argument(
		'-w', '--webapp', dest='webapp', action='store_true',
		help='Configures IQRF Gateway Webapp for target platform.'
	)
	parser.add_argument(
		'-r', '--restart', dest='restart', action='store_true',
		help='Service is restarted after configuration. Has no effect unless Controller or Daemon option is used.'
	)
	parser.add_argument('-v', '--version', action='version', version='%(prog)s v' + __version__)

	args = parser.parse_args()

	sentry_sdk.init(
		"https://96cafcb5cf7644e38daf445dc0b862c4@sentry.iqrf.org/7",
		traces_sample_rate=1.0,
		integrations=[
			AtexitIntegration(AuxiliaryFunctions.atexit_callback())
		]
	)

	logger = Logging.get_logger()
	logger.info('IQRF Gateway Setter v%s', __version__)

	setter_flag = os.path.exists('/var/cache/iqrf-gateway-setter')
	gateway_force = args.gateway and (not setter_flag or args.force)

	try:
		# read gateway config
		gateway_config = GatewayFile.read()

		# configure controller
		if args.controller or gateway_force:
			try:
				ControllerConfig.backup()
				ControllerConfig.configure(gateway_config['gwImage'], gateway_config['gwProduct'])
				if args.restart:
					Command.restart_service('iqrf-gateway-controller')
			except BackupException as backup_error:
				raise backup_error
			except SetterBaseException as controller_error:
				ControllerConfig.restore()
				raise controller_error

		# configure daemon
		if args.daemon or gateway_force:
			try:
				DaemonConfig.backup()
				DaemonConfig.configure_interface(gateway_config['gwProduct'], gateway_config['gwInterface'])
				if args.restart:
					Command.restart_service('iqrf-gateway-daemon')
			except BackupException as backup_error:
				raise backup_error
			except SetterBaseException as daemon_error:
				DaemonConfig.restore()
				raise daemon_error

		# configure uploader
		if args.uploader or gateway_force:
			try:
				UploaderConfig.backup()
				UploaderConfig.configure(gateway_config['gwProduct'])
			except BackupException as backup_error:
				raise backup_error
			except SetterBaseException as uploader_error:
				UploaderConfig.restore()
				raise uploader_error

		# configure webapp
		if args.webapp:
			try:
				WebappConfig.configure(gateway_config)
			except SetterBaseException as webapp_error:
				raise webapp_error

		# configure gateway
		if args.gateway:
			if gateway_force:
				GatewayConfig.configure(gateway_config)
			else:
				logger.info('IQRF Gateway device already configured.')
	except SetterBaseException as setter_error:
		return setter_error.code

	return ErrorCodes.OK


if __name__ == '__main__':
	sys.exit(main())
